#include <gb/gb.h>
#include <gb/font.h>
#include <stdio.h>
#include <rand.h>
#include "acfc_data.c"
#include "acfc_map.c"
#include "grass_data.c"
#include "grass_map.c"
#include "GameCharacter.c"
#include "GameSprites.c"

/*
Created by Logan Leonard for the GBJAM7, August 2019

This wouldn't exist if not for the sample code and tutorial videos created by GingeMonster/GamingMonster

Sample Code: https://github.com/gingemonster/GamingMonstersGameBoySampleCode
GamingMonsters Yotube Channel: https://www.youtube.com/channel/UCMMhSfBStEti-Lqzs30HYWw/videos

Other Tools Used:
GameBoyPngConverter: https://github.com/gingemonster/GameBoyPngConverter
Harry Mulder's Gameboy Tile Designer: http://www.devrs.com/gb/hmgd/gbtd.html

Animal Crossing logo/name/art owned by Nintendo.
*/

struct GameCharacter basket;
struct GameCharacter apple;
struct GameCharacter cherry;
struct GameCharacter pear;

UBYTE spritesize = 8;

void performancedelay(UINT8 numloops){
    UINT8 i;
    for(i = 0; i < numloops; i++){
        wait_vbl_done();
    }
}

UBYTE checkcollisions(struct GameCharacter* one, struct GameCharacter* two){
    return (one->x >= two->x && one->x <= two->x + two->width) && (one->y >= two->y && one->y <= two->y + two->height) || (two->x >= one->x && two->x <= one->x + one->width) && (two->y >= one->y && two->y <= one->y + one->height);
}

void movebasket(struct GameCharacter* character, UINT8 x, UINT8 y){
    move_sprite(character->spritids[0], x, y);
    move_sprite(character->spritids[1], x + spritesize, y);
}

void movegamecharacter(struct GameCharacter* character, UINT8 x, UINT8 y){
    move_sprite(character->spritids[0], x, y);
    move_sprite(character->spritids[1], x + spritesize, y);
    move_sprite(character->spritids[2], x, y + spritesize);
    move_sprite(character->spritids[3], x + spritesize, y + spritesize);
}

void setupkasket(){
    basket.x = 80;
    basket.y = 130;
    basket.width = 16;
    basket.height = 8;

    set_sprite_tile(0, 0);
    basket.spritids[0] = 0;
    set_sprite_tile(1, 1);
    basket.spritids[1] = 1;

    movebasket(&basket, basket.x, basket.y);
}

void setupapple(){
    apple.x = rand()%100;
    apple.y = 0;
    apple.width = 16;
    apple.height = 16;    

    set_sprite_tile(2, 2);
    apple.spritids[0] = 2;
    set_sprite_tile(3, 3);
    apple.spritids[1] = 3;
    set_sprite_tile(4, 4);
    apple.spritids[2] = 4;
    set_sprite_tile(5, 5);
    apple.spritids[3] = 5;

    movegamecharacter(&apple, apple.x, apple.y);
}

void setuppear(){
    pear.x = rand()%100;
    pear.y = 0;
    pear.width = 16;
    pear.height = 16;    

    set_sprite_tile(6, 6);
    pear.spritids[0] = 6;
    set_sprite_tile(7, 7);
    pear.spritids[1] = 7;
    set_sprite_tile(8, 8);
    pear.spritids[2] = 8;
    set_sprite_tile(9, 9);
    pear.spritids[3] = 9;

    movegamecharacter(&pear, pear.x, pear.y);
}

void setupcherry(){
    cherry.x = rand()%100;
    cherry.y = 0;
    cherry.width = 16;
    cherry.height = 16;    

    set_sprite_tile(10, 10);
    cherry.spritids[0] = 10;
    set_sprite_tile(11, 11);
    cherry.spritids[1] = 11;
    set_sprite_tile(12, 12);
    cherry.spritids[2] = 12;
    set_sprite_tile(13, 13);
    cherry.spritids[3] = 13;

    movegamecharacter(&cherry, cherry.x, cherry.y);
}

void bloop(){
    NR21_REG = 0xC1;
    NR22_REG = 0xF1;
    NR23_REG = 0xD6;
    NR24_REG = 0x86;
}

void main(){
    NR52_REG = 0x80;
    NR50_REG = 0x77;
    NR51_REG = 0xFF;

    set_bkg_data(0, 227, acfc_data);
    set_bkg_tiles(0, 0, 20, 18, acfc_map);
    SHOW_BKG;
    DISPLAY_ON;
    waitpad(J_START);
    DISPLAY_OFF;
    set_bkg_data(0, 220, grass_data);
    set_bkg_tiles(0, 0, 20, 18, grass_map);
    DISPLAY_ON;

    set_sprite_data(0, 14, GameSprites);
    setupkasket();
    setupapple();
    setuppear();
    setupcherry();
    SHOW_SPRITES;

    while(1){

        if(joypad() & J_LEFT){
           basket.x -= 2;
           movebasket(&basket, basket.x, basket.y);
       }

       if(joypad() & J_RIGHT){
           basket.x += 2;
           movebasket(&basket, basket.x, basket.y);
       }

       if(checkcollisions(&basket, &apple)){
           apple.y=0;
           apple.x = rand()%100;
           apple.y += 1;
           bloop();
       }
       else
       {
           apple.y += 1;
           if(apple.y >= 170){
               apple.y=0;
               apple.x = rand()%100;
               }
       movegamecharacter(&apple,apple.x,apple.y);
       }
       
       if(checkcollisions(&basket, &pear)){
           pear.y=0;
           pear.x = rand()%100;
           pear.y += 1;
           bloop();
       }
       else
       {
           pear.y += 1;
           if(pear.y >= 170){
               pear.y=-0;
               pear.x = rand()%100;
               }
        movegamecharacter(&pear,pear.x,pear.y);
       }

       if(checkcollisions(&basket, &cherry)){
           cherry.y=0;
           cherry.x = rand()%100;
           cherry.y += 1;
           bloop();
       }
       else
       {
           cherry.y += 2;
           if(cherry.y >= 170){
               cherry.y=-0;
               cherry.x = rand()%100;
               }
        movegamecharacter(&cherry,cherry.x,cherry.y);
       }
       performancedelay(1);
    }
}